import * as i0 from '@angular/core';
import { EventEmitter, Directive, Inject, HostBinding, ContentChildren, forwardRef, Input, Output, HostListener, NgModule } from '@angular/core';
import * as i2 from '@mintplayer/ng-swiper/observe-size';
import { BsObserveSizeDirective } from '@mintplayer/ng-swiper/observe-size';
import { combineLatest, map, delay, filter, mergeMap, debounceTime, BehaviorSubject, take } from 'rxjs';
import { DOCUMENT, CommonModule } from '@angular/common';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import * as i1 from '@angular/animations';
import { style, animate } from '@angular/animations';

class BsSwipeContainerDirective {
    animationBuilder;
    observeSize;
    constructor(element, animationBuilder, document, observeSize) {
        this.animationBuilder = animationBuilder;
        this.observeSize = observeSize;
        this.containerElement = element;
        this.document = document;
        this.offset$ = combineLatest([this.startTouch$, this.lastTouch$, this.imageIndex$, this.isViewInited$])
            .pipe(map(([startTouch, lastTouch, imageIndex, isViewInited]) => {
            if (!isViewInited) {
                return (-imageIndex * 100);
            }
            else if (!!startTouch && !!lastTouch) {
                return (-imageIndex * 100 + (lastTouch.position.x - startTouch.position.x) / this.containerElement.nativeElement.clientWidth * 100);
            }
            else {
                return (-imageIndex * 100);
            }
        }));
        this.padLeft$ = this.swipes$.pipe(map(swipes => {
            if (!swipes) {
                return 0;
            }
            let count = 0;
            for (const s of swipes) {
                if (!s.offside) {
                    break;
                }
                else {
                    count++;
                }
            }
            return count;
        }));
        this.padRight$ = this.swipes$.pipe(map(swipes => {
            if (!swipes) {
                return 0;
            }
            let count = 0;
            for (const s of swipes.toArray().reverse()) {
                if (!s.offside) {
                    break;
                }
                else {
                    count++;
                }
            }
            return count;
        }));
        this.offsetLeft$ = combineLatest([this.offset$, this.padLeft$])
            .pipe(map(([offset, padLeft]) => offset - padLeft * 100));
        this.offsetRight$ = combineLatest([this.offset$, this.padLeft$, this.padRight$])
            .pipe(map(([offset, padLeft, padRight]) => -(offset - padLeft * 100) - (padRight - 1) * 100));
        this.offsetLeft$.pipe(takeUntilDestroyed())
            .subscribe(offsetLeft => this.offsetLeft = offsetLeft);
        this.offsetRight$.pipe(takeUntilDestroyed())
            .subscribe(offsetRight => this.offsetRight = offsetRight);
        this.imageIndex$.pipe(takeUntilDestroyed())
            .subscribe(imageIndex => this.imageIndexChange.emit(imageIndex));
        this.actualSwipes$ = this.swipes$
            .pipe(map(swipes => {
            if (swipes) {
                return swipes.filter(swipe => !swipe.offside);
            }
            else {
                return [];
            }
        }));
        this.slideSizes$ = this.actualSwipes$
            .pipe(delay(400), filter(swipes => !!swipes))
            .pipe(mergeMap(swipes => combineLatest(swipes.map(swipe => swipe.observeSize.size$))));
        this.currentSlideHeight$ = combineLatest([this.slideSizes$, this.imageIndex$])
            .pipe(map(([slideSizes, imageIndex]) => {
            const maxHeight = Math.max(...slideSizes.map(s => s?.height ?? 1));
            console.log('maxHeight', { maxHeight, slideSizes });
            const currHeight = slideSizes[imageIndex]?.height ?? maxHeight;
            // switch (orientation) {
            //   case 'horizontal':
            return currHeight;
            //   case 'vertical':
            //     return maxHeight;
            // }
        }))
            .pipe(debounceTime(10));
    }
    offsetLeft = null;
    offsetRight = null;
    offsetTop = null;
    offsetBottom = null;
    set swipes(value) {
        setTimeout(() => this.swipes$.next(value));
    }
    minimumOffset = 50;
    //#region ImageIndex
    get imageIndex() {
        return this.imageIndex$.value;
    }
    set imageIndex(value) {
        this.imageIndex$.next(value);
    }
    imageIndexChange = new EventEmitter();
    //#endregion
    actualSwipes$;
    isViewInited$ = new BehaviorSubject(false);
    startTouch$ = new BehaviorSubject(null);
    lastTouch$ = new BehaviorSubject(null);
    swipes$ = new BehaviorSubject(null);
    // TODO: slide sizes instead
    slideSizes$;
    imageIndex$ = new BehaviorSubject(0);
    currentSlideHeight$;
    pendingAnimation;
    containerElement;
    document;
    // TODO: Don't just keep px, but both px and % using currentslidesize$
    offset$;
    offsetLeft$;
    offsetRight$;
    padLeft$;
    padRight$;
    ngAfterViewInit() {
        this.isViewInited$.next(true);
    }
    animateToIndexByDx(dx) {
        combineLatest([this.imageIndex$, this.actualSwipes$]).pipe(take(1))
            .subscribe(([imageIndex, actualSwipes]) => {
            const direction = dx > 0 ? 'left' : 'right';
            let newIndex;
            if (Math.abs(dx) < this.minimumOffset) {
                newIndex = imageIndex;
            }
            else {
                newIndex = imageIndex + (direction === 'right' ? 1 : -1);
            }
            this.animateToIndex(imageIndex, newIndex, dx, actualSwipes?.length ?? 1);
        });
    }
    animateToIndex(oldIndex, newIndex, dx, totalSlides) {
        this.pendingAnimation = this.animationBuilder.build([
            style({ 'margin-left': (-(oldIndex + 1) * this.containerElement.nativeElement.clientWidth + dx) + 'px', 'margin-right': ((oldIndex + 1) * this.containerElement.nativeElement.clientWidth - dx) + 'px' }),
            animate('500ms ease', style({ 'margin-left': (-(newIndex + 1) * this.containerElement.nativeElement.clientWidth) + 'px', 'margin-right': ((newIndex + 1) * this.containerElement.nativeElement.clientWidth) + 'px' })),
        ]).create(this.containerElement.nativeElement);
        this.pendingAnimation.onDone(() => {
            // Correct the image index
            if (newIndex === -1) {
                this.imageIndex$.next(totalSlides - 1);
            }
            else if (newIndex === totalSlides) {
                this.imageIndex$.next(0);
            }
            else {
                this.imageIndex$.next(newIndex);
            }
            this.startTouch$.next(null);
            this.lastTouch$.next(null);
            this.pendingAnimation?.destroy();
            this.pendingAnimation = undefined;
        });
        this.pendingAnimation.play();
    }
    onSwipe(dx) {
        this.animateToIndexByDx(dx);
    }
    previous() {
        this.gotoAnimate(-1, 'relative');
    }
    next() {
        this.gotoAnimate(1, 'relative');
    }
    goto(index) {
        this.gotoAnimate(index, 'absolute');
    }
    gotoAnimate(index, type) {
        this.pendingAnimation?.finish();
        setTimeout(() => {
            combineLatest([this.actualSwipes$, this.imageIndex$]).pipe(take(1)).subscribe(([actualSwipes, imageIndex]) => {
                this.pendingAnimation?.finish();
                const idx = (type === 'relative') ? imageIndex + index : index;
                this.animateToIndex(imageIndex, idx, 0, actualSwipes?.length ?? 1);
            });
        }, 20);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwipeContainerDirective, deps: [{ token: i0.ElementRef }, { token: i1.AnimationBuilder }, { token: DOCUMENT }, { token: i2.BsObserveSizeDirective }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.3", type: BsSwipeContainerDirective, isStandalone: false, selector: "[bsSwipeContainer]", inputs: { minimumOffset: "minimumOffset", imageIndex: "imageIndex" }, outputs: { imageIndexChange: "imageIndexChange" }, host: { properties: { "style.margin-left.%": "this.offsetLeft", "style.margin-right.%": "this.offsetRight", "style.margin-top.%": "this.offsetTop", "style.margin-bottom.%": "this.offsetBottom" } }, queries: [{ propertyName: "swipes", predicate: i0.forwardRef(() => BsSwipeDirective) }], exportAs: ["bsSwipeContainer"], hostDirectives: [{ directive: i2.BsObserveSizeDirective }], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwipeContainerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bsSwipeContainer]',
                    exportAs: 'bsSwipeContainer',
                    standalone: false,
                    hostDirectives: [BsObserveSizeDirective],
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i1.AnimationBuilder }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: i2.BsObserveSizeDirective }], propDecorators: { offsetLeft: [{
                type: HostBinding,
                args: ['style.margin-left.%']
            }], offsetRight: [{
                type: HostBinding,
                args: ['style.margin-right.%']
            }], offsetTop: [{
                type: HostBinding,
                args: ['style.margin-top.%']
            }], offsetBottom: [{
                type: HostBinding,
                args: ['style.margin-bottom.%']
            }], swipes: [{
                type: ContentChildren,
                args: [forwardRef(() => BsSwipeDirective)]
            }], minimumOffset: [{
                type: Input
            }], imageIndex: [{
                type: Input
            }], imageIndexChange: [{
                type: Output
            }] } });

class BsSwipeDirective {
    container;
    constructor(container, observeSize) {
        this.container = container;
        this.observeSize = observeSize;
        // container.orientation$.pipe(takeUntilDestroyed())
        //   .subscribe(orientation => this.hostClass = (orientation === 'vertical') ? 'd-block' : 'd-inline-block');
    }
    observeSize;
    //#region Offside
    offside = false;
    //#endregion
    classes = true;
    hostClass;
    onTouchStart(ev) {
        if (ev.touches.length === 1) {
            ev.preventDefault();
            this.container.pendingAnimation?.finish();
            setTimeout(() => {
                this.container.startTouch$.next({
                    position: {
                        x: ev.touches[0].clientX,
                        y: ev.touches[0].clientY,
                    },
                    timestamp: Date.now(),
                });
                this.container.lastTouch$.next({
                    position: {
                        x: ev.touches[0].clientX,
                        y: ev.touches[0].clientY,
                    },
                    isTouching: true,
                });
            }, 20);
        }
    }
    onTouchMove(ev) {
        this.container.lastTouch$.next({
            position: {
                x: ev.touches[0].clientX,
                y: ev.touches[0].clientY,
            },
            isTouching: true,
        });
    }
    onTouchEnd(ev) {
        combineLatest([this.container.startTouch$, this.container.lastTouch$])
            .pipe(filter(([startTouch, lastTouch]) => !!startTouch && !!lastTouch))
            .pipe(take(1))
            .subscribe(([startTouch, lastTouch]) => {
            if (!!startTouch && !!lastTouch) {
                const dx = lastTouch.position.x - startTouch.position.x;
                this.container.onSwipe(dx);
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwipeDirective, deps: [{ token: BsSwipeContainerDirective }, { token: i2.BsObserveSizeDirective }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.0.3", type: BsSwipeDirective, isStandalone: false, selector: "[bsSwipe]", inputs: { offside: "offside" }, host: { listeners: { "touchstart": "onTouchStart($event)", "touchmove": "onTouchMove($event)", "touchend": "onTouchEnd($event)" }, properties: { "class.align-top": "this.classes", "class.d-inline-block": "this.classes", "class.float-none": "this.classes", "class.w-100": "this.classes", "class.pe-auto": "this.classes", "class.me-0": "this.classes", "class": "this.hostClass" } }, hostDirectives: [{ directive: i2.BsObserveSizeDirective }], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwipeDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bsSwipe]',
                    hostDirectives: [BsObserveSizeDirective],
                    standalone: false,
                }]
        }], ctorParameters: () => [{ type: BsSwipeContainerDirective }, { type: i2.BsObserveSizeDirective }], propDecorators: { offside: [{
                type: Input
            }], classes: [{
                type: HostBinding,
                args: ['class.align-top']
            }, {
                type: HostBinding,
                args: ['class.d-inline-block']
            }, {
                type: HostBinding,
                args: ['class.float-none']
            }, {
                type: HostBinding,
                args: ['class.w-100']
            }, {
                type: HostBinding,
                args: ['class.pe-auto']
            }, {
                type: HostBinding,
                args: ['class.me-0']
            }], hostClass: [{
                type: HostBinding,
                args: ['class']
            }], onTouchStart: [{
                type: HostListener,
                args: ['touchstart', ['$event']]
            }], onTouchMove: [{
                type: HostListener,
                args: ['touchmove', ['$event']]
            }], onTouchEnd: [{
                type: HostListener,
                args: ['touchend', ['$event']]
            }] } });

class BsSwiperModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwiperModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.0.3", ngImport: i0, type: BsSwiperModule, declarations: [BsSwipeDirective, BsSwipeContainerDirective], imports: [CommonModule], exports: [BsSwipeDirective, BsSwipeContainerDirective] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwiperModule, imports: [CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.3", ngImport: i0, type: BsSwiperModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [BsSwipeDirective, BsSwipeContainerDirective],
                    exports: [BsSwipeDirective, BsSwipeContainerDirective],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BsSwipeContainerDirective, BsSwipeDirective, BsSwiperModule };
//# sourceMappingURL=mintplayer-ng-swiper-swiper.mjs.map
